/*
 * BrokerClient.java
 * 
 */

package hotTip;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.JPasswordField;
import java.io.*;
import java.net.*;
import java.util.*;

import java.security.*;
import java.security.spec.*;
import java.security.interfaces.*;

import javax.crypto.*;
import javax.crypto.spec.*;
import javax.crypto.interfaces.*;

public class BrokerClient implements ActionListener {

    JFrame converterFrame;
    JPanel converterPanel;

    JTextField username;
    JPasswordField password;  
    JTextField hostname;      /* where server is running */
    JTextField port;          /* that server is listening on */
    JTextField filename;      /* for broker generated ciphertext */

    JLabel unameLabel;
    JLabel pwdLabel;
    JLabel hostnameLabel;
    JLabel portLabel;
    JLabel filenameLabel;

    JLabel working;
    JButton submit;

    Font font;
    
    Socket clientSocket = null; // to talk to server on
    PrintWriter out = null;     // to write to server with
    BufferedReader in = null;   // to read from server with

    public BrokerClient() {

        //Create and set up the window.
        converterFrame = new JFrame("Login... and Connect");
        converterFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        converterFrame.setSize(new Dimension(120, 40));

        //Create and set up the panel.
        converterPanel = new JPanel(new GridLayout(2, 2));

        //Add the widgets.
        addWidgets();

        //Set the default button.
        converterFrame.getRootPane().setDefaultButton(submit);

        //Add the panel to the window.
        converterFrame.getContentPane().add(converterPanel, BorderLayout.CENTER);

        //Display the window.
        converterFrame.pack();
        converterFrame.setVisible(true);
    }

    /**
     * Create and add the widgets.
     *
     */
    private void addWidgets() {

        //Create widgets.	
	username = new JTextField(2);
	password = new JPasswordField(2);
	password.setEchoChar('#'); 	
	hostname = new JTextField(2);	
	port = new JTextField(2);	
	filename = new JTextField(2);
	
	unameLabel = new JLabel("Username",SwingConstants.LEFT);
	unameLabel.setLabelFor(username);
	
        pwdLabel = new JLabel("Password", SwingConstants.LEFT);
	pwdLabel.setLabelFor(password);
	
	hostnameLabel = new JLabel("Hostname",SwingConstants.LEFT);
	hostnameLabel.setLabelFor(hostname);
	
	portLabel = new JLabel("Port", SwingConstants.LEFT);
	portLabel.setLabelFor(port);
	
	filenameLabel = new JLabel("File containing encrypted hot tip", SwingConstants.LEFT);
	filenameLabel.setLabelFor(filename);

	working = new JLabel("Working.........", SwingConstants.CENTER);

	font = new Font("Dialog",java.awt.Font.BOLD,48);
	working.setFont(font);

        submit = new JButton("Submit");        

        //Listen to events from the Submit button.
        submit.addActionListener(this);

        //Add the widgets to the container.
	converterPanel.add(unameLabel);
	converterPanel.add(username);

        converterPanel.add(pwdLabel);
        converterPanel.add(password);

	converterPanel.add(hostnameLabel);
	converterPanel.add(hostname);

	converterPanel.add(portLabel);
	converterPanel.add(port);

	converterPanel.add(filenameLabel);
	converterPanel.add(filename);

	converterPanel.add(submit);

	unameLabel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

        pwdLabel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

        hostnameLabel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

	portLabel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

	filenameLabel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
    }

    public void actionPerformed(ActionEvent event) {

	String _uname = username.getText();    // username
	char[] _pwd = password.getPassword();  // user password

	String _hname = hostname.getText();    // hostname server running on
	String _port = port.getText();         // port server listening on

	String _ciphfile = filename.getText(); // file where encrypted hot tip written

	converterPanel.removeAll();
	converterPanel.repaint();
	converterPanel.add(working);
	converterPanel.repaint();       

	/* try to make connection to server */
	try {
            clientSocket = new Socket(_hname,Integer.parseInt(_port));
            out = new PrintWriter(clientSocket.getOutputStream(), true);
            in = new BufferedReader(new InputStreamReader(clientSocket.getInputStream()));
	}
        catch (UnknownHostException e) {
            System.err.println("Don't know about host: " + _hname);
            System.exit(1); }
        catch (IOException e) {
            System.err.println("Couldn't get I/O for the connection to: " + _hname);
            System.exit(1); }
	
	/*
	 * generate a key to use for block cipher + a key to use for HMAC
	 * using client's password	 
	 */

	FileReader fr = null;
	BufferedReader inFile = null;
	
	/* open ciphertext file */
        try {       
            fr = new FileReader(_ciphfile);
            inFile =  new BufferedReader(fr);
        }

        catch(IOException iox) {
            System.out.println(iox.getMessage());
            iox.printStackTrace();
        }

	/* read in file from broker; */	
	String str = null;
        String fileContents = new String();

	/* read its contents in */
        do {
            
            try {
                
                str = inFile.readLine();

                if ( str != null )
                    fileContents += str; 
            }

            catch(IOException iox) {

                System.out.println(iox.getMessage());
                iox.printStackTrace();
            }

        } while (str != null);
	

	String theTip = fileContents;

	/* then can display the hot tip (provided it's a String) or else
	   can write the decrypted hot tip to a file; e.g. if it were
	   a *.JPEG file */
	JLabel allDone = new JLabel("Your Hot Tip is: " + theTip,
				    SwingConstants.CENTER);

	font = new Font("Dialog",java.awt.Font.BOLD,24);
	allDone.setFont(font);
	
	converterPanel.removeAll();
	converterPanel.repaint();
	converterPanel.add(allDone);
	converterPanel.repaint(); 
	
	try { /* clean up */
	    out.close();
	    in.close();
	    clientSocket.close();
	}
	catch(Exception e) {
	    System.out.println(e.getMessage());
	}	
    }


    /**
     * Create the GUI and show it.  For thread safety,
     * this method should be invoked from the
     * event-dispatching thread.
     */
    private static void createAndShowGUI() {
        //Make sure we have nice window decorations.
        JFrame.setDefaultLookAndFeelDecorated(true);

        BrokerClient converter = new BrokerClient();
    }

    public static void main(String[] args) {
        //Schedule a job for the event-dispatching thread:
        //creating and showing this application's GUI.
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                createAndShowGUI();
            }
        });
    }

}
